#include "main.h"

VOID onUnload(_In_ struct _DRIVER_OBJECT* DriverObject) {
    UNREFERENCED_PARAMETER(DriverObject);
    KdPrint(("[yhy]: %s", __FUNCDNAME__));
    UNICODE_STRING symLink;
    RtlInitUnicodeString(&symLink, SYMBOLIC_LINK);
    NTSTATUS status = IoDeleteSymbolicLink(&symLink);  // 删除符号链接
    if (!NT_SUCCESS(status)) {
        KdPrint(("[yhy] [Error]: IoDeleteSymbolicLink Error! rrorCode:%08x\n", status));
        return;
    }
    // 删除设备
    if (DriverObject->DeviceObject != NULL) {
        IoDeleteDevice(DriverObject->DeviceObject);
    }
    KdPrint(("[yhy]: onUnload Success!\n"));
    KdPrint(("[yhy]: Bye!\n"));
}

NTSTATUS DispatchCreate(_In_ struct _DEVICE_OBJECT* DeviceObject, _Inout_ struct _IRP* Irp) {
    NTSTATUS status = STATUS_SUCCESS;
    KdPrint(("[yhy]: %s\n", __FUNCDNAME__));
    IoCompleteRequest(Irp, IO_NO_INCREMENT);
    UNREFERENCED_PARAMETER(DeviceObject);
    return status;
}

NTSTATUS DispatchClose(_In_ struct _DEVICE_OBJECT* DeviceObject, _Inout_ struct _IRP* Irp) {
    NTSTATUS status = STATUS_SUCCESS;
    KdPrint(("[yhy]: %s\n", __FUNCDNAME__));
    IoCompleteRequest(Irp, IO_NO_INCREMENT);
    UNREFERENCED_PARAMETER(DeviceObject);
    Irp->IoStatus.Status = STATUS_SUCCESS;
    Irp->IoStatus.Information = 0;
    return status;
}
NTSTATUS DispatchRead(_In_ struct _DEVICE_OBJECT* DeviceObject, _Inout_ struct _IRP* Irp) {
    NTSTATUS status = STATUS_SUCCESS;
    KdPrint(("[yhy]: %s\n", __FUNCDNAME__));
    IoCompleteRequest(Irp, IO_NO_INCREMENT);
    UNREFERENCED_PARAMETER(DeviceObject);
    return status;
}
NTSTATUS DispatchWrite(_In_ struct _DEVICE_OBJECT* DeviceObject, _Inout_ struct _IRP* Irp) {
    NTSTATUS status = STATUS_SUCCESS;
    KdPrint(("[yhy]: %s\n", __FUNCDNAME__));
    IoCompleteRequest(Irp, IO_NO_INCREMENT);
    UNREFERENCED_PARAMETER(DeviceObject);
    return status;
}

NTSTATUS DispatchControl(_In_ struct _DEVICE_OBJECT* DeviceObject, _Inout_ struct _IRP* Irp) {
    NTSTATUS status = STATUS_SUCCESS;
    UNREFERENCED_PARAMETER(DeviceObject);
    KdPrint(("[yhy]: %s\n", __FUNCDNAME__));
    PIO_STACK_LOCATION pIrpStack = IoGetCurrentIrpStackLocation(Irp);
    if (pIrpStack->Parameters.DeviceIoControl.IoControlCode == IOCTL_READ_MEMORY) {
        PVOID ioBuffer = Irp->AssociatedIrp.SystemBuffer;
        //ULONG bufferLen = max(pIrpStack->Parameters.DeviceIoControl.InputBufferLength, pIrpStack->Parameters.DeviceIoControl.OutputBufferLength);
        // 读取目标内存
        void* targetAddress = *(void**)ioBuffer;
        KdPrint(("[yhy]: DispatchControl ioBuffer:%p targetAddress:%p\n", ioBuffer, targetAddress));
        int newValue = 0x12341234;
        RtlCopyMemory(ioBuffer, &newValue, 4);
        Irp->IoStatus.Status = STATUS_SUCCESS;
        Irp->IoStatus.Information = 4;
    }
    IoCompleteRequest(Irp, IO_NO_INCREMENT);
    return status;
}

NTSTATUS DriverEntry(
    _In_ PDRIVER_OBJECT     DriverObject,
    _In_ PUNICODE_STRING    RegistryPath
)
{
    UNREFERENCED_PARAMETER(RegistryPath);
    // NTSTATUS variable to record success or failure
    NTSTATUS status = STATUS_SUCCESS;
    DriverObject->DriverUnload = onUnload;
    // 1. 绑定IRP 处理Dispatch例程
    DriverObject->MajorFunction[IRP_MJ_CREATE] = DispatchCreate;
    DriverObject->MajorFunction[IRP_MJ_CLOSE] = DispatchClose;
    DriverObject->MajorFunction[IRP_MJ_READ] = DispatchRead;
    DriverObject->MajorFunction[IRP_MJ_WRITE] = DispatchWrite;
    DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL] = DispatchControl;
    // 2. 创建设备对象
    UNICODE_STRING deviceName;
    PDEVICE_OBJECT pDevice = NULL;
    RtlInitUnicodeString(&deviceName, L"\\Device\\" MY_DRVICE_NAME);
    KdPrint(("[yhy] deviceName: %wZ", &deviceName));
    status = IoCreateDevice(DriverObject, 0, &deviceName, FILE_DEVICE_UNKNOWN, FILE_DEVICE_SECURE_OPEN, FALSE /*非独占*/,
        &pDevice);
    if (!NT_SUCCESS(status)) {
        KdPrint(("[yhy] DriverEntry IoCreateDevice ErrorCode:%08x\n", status));
        return status;
    }
    // 3. 创建符号
    UNICODE_STRING symbolName;
    // L"\\DosDevices\\MY_DRVICE_NAME"
    RtlInitUnicodeString(&symbolName, SYMBOLIC_LINK);
    status = IoCreateSymbolicLink(&symbolName, &deviceName);
    if (!NT_SUCCESS(status)) {
        KdPrint(("[yhy] DriverEntry IoCreateSymbolicLink ErrorCode:%08x\n", status));
        return status;
    }
    KdPrint(("[yhy] DriverEntry IoCreateSymbolicLink Success! symbolName is :%wZ", &symbolName));
    return status;
}